from torch import Tensor
from pege.egnn import model
from pege.utils import pdb2feats


class Pege:
    def __init__(self, path: str):
        """ """
        self.path = path
        self.coords, self.feats, self.anumbs, self.details = pdb2feats(path)
        self.embs = model(self.coords, self.feats).squeeze()

    def get_protein(self, pooling: str = "avg") -> Tensor:
        """ """
        protein_emb = self.embs
        if pooling:
            protein_emb = self.apply_pool(protein_emb, pooling)
        return protein_emb

    def get_atoms(
        self, atom_numbers: list, ignore_missing: bool = False, pooling: str = "avg"
    ) -> Tensor:
        """ """
        atom_is = []
        if ignore_missing:
            for i in atom_numbers:
                if i not in self.anumbs:
                    raise Exception(f"Atom {i} not present")
                a_i = self.anumbs.index(i)
                atom_is.append(a_i)
        pocket_embs = self.embs[atom_is]
        if pooling:
            pocket_embs = self.apply_pool(pocket_embs, pooling)
        return pocket_embs

    @staticmethod
    def apply_pool(t: Tensor, ptype: str) -> Tensor:
        if ptype == "avg":
            t = t.mean(dim=0)
        elif ptype == "sum":
            t = t.sum(dim=0)
        return t



import torch
import os
from pytorch_lightning.core.lightning import LightningModule
from egnn_pytorch import EGNN_Network


class EGNNLayer(LightningModule):
    def __init__(self):
        super().__init__()

        self.egnn = EGNN_Network(
            num_tokens=18,
            dim=64,
            depth=3,
            num_nearest_neighbors=32,
            norm_coors=True,
            coor_weights_clamp_value=2.0,
        )

    def forward(self, coords, feats):
        feats = feats.int().to(self.device)
        coords = coords.float().to(self.device)
        mask = torch.ones_like(feats).bool().to(self.device)

        feats_out, _ = self.egnn(feats, coords, mask=mask)
        return feats_out


model = EGNNLayer()
curdir = os.path.dirname(os.path.realpath(__file__))
model.load_state_dict(torch.load(f"{curdir}/pege.pth"))



import torch

OHE_ATOMS_GRAPH = [
    "H",
    "N",
    "NZ_LYS",
    "NH_ARG",
    "O_AMIDE",
    "OH_TYR",
    "NE2_HIS",
    "SD_MET",
    "SG_CYS",
    "OG_SER",
    "O",
    "NE1_TRP",
    "OG1_THR",
    "O_COOH",
    "NE_ARG",
    "N_AMIDE",
    "ND1_HIS",
]


def read_pdb_line(line: str) -> list:
    aname = line[12:16].strip()
    anumb = int(line[5:11].strip())
    resname = line[17:21].strip()
    chain = line[21]
    resnumb = int(line[22:26])
    x = float(line[30:38])
    y = float(line[38:46])
    z = float(line[46:54])
    return (aname, anumb, resname, chain, resnumb, x, y, z)


def classify_atom(aname: str, resname: str) -> str:
    if aname in ("N", "O"):
        if resname != "NTR":
            return aname
        else:
            return "NZ_LYS"
    elif resname in ("CTR", "ASP", "GLU"):
        return f"O_COOH"
    elif resname in ("GLN", "ASN"):
        return f"{aname[0]}_AMIDE"
    elif resname == "ARG" and aname in ("NH1", "NH2"):
        return "NH_ARG"
    elif resname == "CY0" and aname == "SG":
        return "SG_CYS"
    else:
        return f"{aname}_{resname}"


def pdb2feats(fname: str):
    def iter_f(fname):
        with open(fname) as f:
            for line in f:
                yield line

    coords, feats, anumbs, details = [], [], [], []
    for line in iter_f(fname):
        if not line.startswith("ATOM "):
            continue

        aname, anumb, resname, chain, resnumb, x, y, z = read_pdb_line(line)
        res = classify_atom(aname, resname)

        if res in OHE_ATOMS_GRAPH:
            res_ohe = OHE_ATOMS_GRAPH.index(res)

            coords.append((x, y, z))
            feats.append(res_ohe)
            anumbs.append(anumb)
            details.append((chain, resnumb, resname, aname))

    coords = torch.tensor(coords).reshape(1, -1, 3)
    feats = torch.tensor(feats).reshape(1, -1)

    return coords, feats, anumbs, details
